﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

namespace AttachedBehaviors.Framework
{
	public class EnumCheck
	{
		private readonly List<object> _parsedTargetValues = new List<object>();

		public EnumCheck()
		{
			ParsedTargetValues = _parsedTargetValues.AsReadOnly();
		}

		public object Value { get; private set; }

		public string TargetValue { get; private set; }

		public ReadOnlyCollection<object> ParsedTargetValues { get; private set; }

		public object ParsedTargetValue
		{
			get { return ParsedTargetValues.FirstOrDefault(); }
		}

		public bool IsMatch { get; private set; }

		public void Update(object value, string targetValue)
		{
			var valueChanged = value != Value;
			var targetValueChanged = targetValue != TargetValue;

			if(valueChanged || targetValueChanged)
			{
				if(valueChanged)
				{
					Value = value;
				}

				if(targetValueChanged)
				{
					TargetValue = targetValue;
				}

				ParseTargetValues();

				MatchValueAndTargetValue();
			}
		}

		private void ParseTargetValues()
		{
			_parsedTargetValues.Clear();

			if(Value != null && Value is Enum && !String.IsNullOrEmpty(TargetValue))
			{
				var parsedValues =
					from targetValue in TargetValue.Split(',')
					let trimmedTargetValue = targetValue.Trim()
					where trimmedTargetValue.Length > 0
					select Enum.Parse(Value.GetType(), trimmedTargetValue, false);

				_parsedTargetValues.AddRange(parsedValues);
			}
		}

		private void MatchValueAndTargetValue()
		{
			IsMatch = Value == null || Value.Equals("")
				? String.IsNullOrEmpty(TargetValue)
				: _parsedTargetValues.Contains(Value);
		}
	}
}