﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using AttachedBehaviors.Framework;

namespace AttachedBehaviors.Usage
{
	public static class EnumGroup
	{
		public static readonly DependencyProperty ValueProperty = DependencyProperty.RegisterAttached(
			"Value",
			typeof(object),
			typeof(EnumGroup),
			new PropertyMetadata(OnArgumentsChanged));

		public static readonly DependencyProperty TargetValueProperty = DependencyProperty.RegisterAttached(
			"TargetValue",
			typeof(string),
			typeof(EnumGroup),
			new PropertyMetadata(OnArgumentsChanged));

		private static readonly AttachedBehavior Behavior =
			AttachedBehavior.Register(host => new EnumGroupBehavior(host));

		private static void OnArgumentsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			Behavior.Update(d);
		}

		public static object GetValue(RadioButton radioButton)
		{
			return radioButton.GetValue(ValueProperty);
		}

		public static void SetValue(RadioButton radioButton, object value)
		{
			radioButton.SetValue(ValueProperty, value);
		}

		public static string GetTargetValue(RadioButton radioButton)
		{
			return (string) radioButton.GetValue(TargetValueProperty);
		}

		public static void SetTargetValue(RadioButton radioButton, string value)
		{
			radioButton.SetValue(TargetValueProperty, value);
		}

		private sealed class EnumGroupBehavior : Behavior<RadioButton>
		{
			private readonly EnumCheck _enumCheck = new EnumCheck();

			internal EnumGroupBehavior(DependencyObject host) : base(host)
			{}

			protected override void Attach(RadioButton host)
			{
				host.Checked += OnChecked;
			}

			protected override void Detach(RadioButton host)
			{
				host.Checked -= OnChecked;
			}

			protected override void Update(RadioButton host)
			{
				_enumCheck.Update(GetValue(host), GetTargetValue(host));

				host.IsChecked = _enumCheck.IsMatch;
			}

			private void OnChecked(object sender, RoutedEventArgs e)
			{
				TryUpdate(host => SetValue(host, _enumCheck.ParsedTargetValue));
			}
		}
	}
}